#ifndef WSJTX_LOGGING_HPP__
#define WSJTX_LOGGING_HPP__

//
// Class WSJTXLogging - wraps application specific logging
//
class WSJTXLogging final
{
public:
  explicit WSJTXLogging ();
  ~WSJTXLogging ();
};

#endif
