module jt65_mod

  integer param(0:9)
  integer mrs(63)
  integer mrs2(63)
  integer mdat(126),mref(126,2),mdat2(126),mref2(126,2)    !From prcom

  real s1(-255:256,126)
  real s3a(64,63)
  real pr(126)
  real width

end module jt65_mod
