# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dw_buttons.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName(_fromUtf8("DockWidget"))
        DockWidget.resize(527, 460)
        self.dockWidgetContents = QtGui.QWidget()
        self.dockWidgetContents.setObjectName(_fromUtf8("dockWidgetContents"))
        self.gridLayout = QtGui.QGridLayout(self.dockWidgetContents)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_72 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_72.setFont(font)
        self.label_72.setObjectName(_fromUtf8("label_72"))
        self.gridLayout.addWidget(self.label_72, 0, 1, 1, 1)
        self.label_73 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_73.setFont(font)
        self.label_73.setObjectName(_fromUtf8("label_73"))
        self.gridLayout.addWidget(self.label_73, 0, 2, 1, 1)
        self.label_26 = QtGui.QLabel(self.dockWidgetContents)
        self.label_26.setMinimumSize(QtCore.QSize(0, 0))
        self.label_26.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_26.setFont(font)
        self.label_26.setObjectName(_fromUtf8("label_26"))
        self.gridLayout.addWidget(self.label_26, 1, 0, 1, 1)
        self.pushButton = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButton.setMinimumSize(QtCore.QSize(0, 0))
        self.pushButton.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.pushButton.setObjectName(_fromUtf8("pushButton"))
        self.gridLayout.addWidget(self.pushButton, 1, 1, 1, 1)
        self.pushButtonDis = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButtonDis.setEnabled(False)
        self.pushButtonDis.setMinimumSize(QtCore.QSize(0, 0))
        self.pushButtonDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.pushButtonDis.setDefault(False)
        self.pushButtonDis.setObjectName(_fromUtf8("pushButtonDis"))
        self.gridLayout.addWidget(self.pushButtonDis, 1, 2, 1, 1)
        self.label_74 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_74.setFont(font)
        self.label_74.setObjectName(_fromUtf8("label_74"))
        self.gridLayout.addWidget(self.label_74, 2, 0, 1, 1)
        self.pushButtonChecked = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButtonChecked.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.pushButtonChecked.setCheckable(True)
        self.pushButtonChecked.setChecked(True)
        self.pushButtonChecked.setObjectName(_fromUtf8("pushButtonChecked"))
        self.gridLayout.addWidget(self.pushButtonChecked, 2, 1, 1, 1)
        self.pushButtonCheckedDis = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButtonCheckedDis.setEnabled(False)
        self.pushButtonCheckedDis.setCheckable(True)
        self.pushButtonCheckedDis.setChecked(True)
        self.pushButtonCheckedDis.setObjectName(_fromUtf8("pushButtonCheckedDis"))
        self.gridLayout.addWidget(self.pushButtonCheckedDis, 2, 2, 1, 1)
        self.label_76 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_76.setFont(font)
        self.label_76.setObjectName(_fromUtf8("label_76"))
        self.gridLayout.addWidget(self.label_76, 3, 0, 1, 1)
        self.pushButtonUnchecked = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButtonUnchecked.setCheckable(True)
        self.pushButtonUnchecked.setObjectName(_fromUtf8("pushButtonUnchecked"))
        self.gridLayout.addWidget(self.pushButtonUnchecked, 3, 1, 1, 1)
        self.pushButtonUncheckedDis = QtGui.QPushButton(self.dockWidgetContents)
        self.pushButtonUncheckedDis.setEnabled(False)
        self.pushButtonUncheckedDis.setCheckable(True)
        self.pushButtonUncheckedDis.setObjectName(_fromUtf8("pushButtonUncheckedDis"))
        self.gridLayout.addWidget(self.pushButtonUncheckedDis, 3, 2, 1, 1)
        self.label_33 = QtGui.QLabel(self.dockWidgetContents)
        self.label_33.setMinimumSize(QtCore.QSize(0, 0))
        self.label_33.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_33.setFont(font)
        self.label_33.setObjectName(_fromUtf8("label_33"))
        self.gridLayout.addWidget(self.label_33, 4, 0, 1, 1)
        self.toolButton = QtGui.QToolButton(self.dockWidgetContents)
        self.toolButton.setMinimumSize(QtCore.QSize(0, 0))
        self.toolButton.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.toolButton.setObjectName(_fromUtf8("toolButton"))
        self.gridLayout.addWidget(self.toolButton, 4, 1, 1, 1)
        self.toolButtonDis = QtGui.QToolButton(self.dockWidgetContents)
        self.toolButtonDis.setEnabled(False)
        self.toolButtonDis.setMinimumSize(QtCore.QSize(0, 0))
        self.toolButtonDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.toolButtonDis.setObjectName(_fromUtf8("toolButtonDis"))
        self.gridLayout.addWidget(self.toolButtonDis, 4, 2, 1, 1)
        self.label_75 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_75.setFont(font)
        self.label_75.setObjectName(_fromUtf8("label_75"))
        self.gridLayout.addWidget(self.label_75, 5, 0, 1, 1)
        self.radioButtonChecked = QtGui.QRadioButton(self.dockWidgetContents)
        self.radioButtonChecked.setChecked(True)
        self.radioButtonChecked.setAutoExclusive(False)
        self.radioButtonChecked.setObjectName(_fromUtf8("radioButtonChecked"))
        self.gridLayout.addWidget(self.radioButtonChecked, 5, 1, 1, 1)
        self.radioButtonCheckedDis = QtGui.QRadioButton(self.dockWidgetContents)
        self.radioButtonCheckedDis.setEnabled(False)
        self.radioButtonCheckedDis.setChecked(True)
        self.radioButtonCheckedDis.setAutoExclusive(False)
        self.radioButtonCheckedDis.setObjectName(_fromUtf8("radioButtonCheckedDis"))
        self.gridLayout.addWidget(self.radioButtonCheckedDis, 5, 2, 1, 1)
        self.label_29 = QtGui.QLabel(self.dockWidgetContents)
        self.label_29.setMinimumSize(QtCore.QSize(0, 0))
        self.label_29.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_29.setFont(font)
        self.label_29.setObjectName(_fromUtf8("label_29"))
        self.gridLayout.addWidget(self.label_29, 6, 0, 1, 1)
        self.radioButtonUnchecked = QtGui.QRadioButton(self.dockWidgetContents)
        self.radioButtonUnchecked.setMinimumSize(QtCore.QSize(0, 0))
        self.radioButtonUnchecked.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.radioButtonUnchecked.setAutoExclusive(False)
        self.radioButtonUnchecked.setObjectName(_fromUtf8("radioButtonUnchecked"))
        self.gridLayout.addWidget(self.radioButtonUnchecked, 6, 1, 1, 1)
        self.radioButtonUncheckedDis = QtGui.QRadioButton(self.dockWidgetContents)
        self.radioButtonUncheckedDis.setEnabled(False)
        self.radioButtonUncheckedDis.setMinimumSize(QtCore.QSize(0, 0))
        self.radioButtonUncheckedDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.radioButtonUncheckedDis.setChecked(False)
        self.radioButtonUncheckedDis.setAutoExclusive(False)
        self.radioButtonUncheckedDis.setObjectName(_fromUtf8("radioButtonUncheckedDis"))
        self.gridLayout.addWidget(self.radioButtonUncheckedDis, 6, 2, 1, 1)
        self.label_53 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_53.setFont(font)
        self.label_53.setObjectName(_fromUtf8("label_53"))
        self.gridLayout.addWidget(self.label_53, 7, 0, 1, 1)
        self.checkBoxChecked = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxChecked.setChecked(True)
        self.checkBoxChecked.setObjectName(_fromUtf8("checkBoxChecked"))
        self.gridLayout.addWidget(self.checkBoxChecked, 7, 1, 1, 1)
        self.checkBoxCheckedDis = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxCheckedDis.setEnabled(False)
        self.checkBoxCheckedDis.setChecked(True)
        self.checkBoxCheckedDis.setObjectName(_fromUtf8("checkBoxCheckedDis"))
        self.gridLayout.addWidget(self.checkBoxCheckedDis, 7, 2, 1, 1)
        self.label_30 = QtGui.QLabel(self.dockWidgetContents)
        self.label_30.setMinimumSize(QtCore.QSize(0, 0))
        self.label_30.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_30.setFont(font)
        self.label_30.setObjectName(_fromUtf8("label_30"))
        self.gridLayout.addWidget(self.label_30, 8, 0, 1, 1)
        self.checkBoxEnabled = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxEnabled.setMinimumSize(QtCore.QSize(0, 0))
        self.checkBoxEnabled.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.checkBoxEnabled.setTristate(False)
        self.checkBoxEnabled.setObjectName(_fromUtf8("checkBoxEnabled"))
        self.gridLayout.addWidget(self.checkBoxEnabled, 8, 1, 1, 1)
        self.checkBoxUncheckedDis = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxUncheckedDis.setEnabled(False)
        self.checkBoxUncheckedDis.setMinimumSize(QtCore.QSize(0, 0))
        self.checkBoxUncheckedDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.checkBoxUncheckedDis.setChecked(False)
        self.checkBoxUncheckedDis.setObjectName(_fromUtf8("checkBoxUncheckedDis"))
        self.gridLayout.addWidget(self.checkBoxUncheckedDis, 8, 2, 1, 1)
        self.label_31 = QtGui.QLabel(self.dockWidgetContents)
        self.label_31.setMinimumSize(QtCore.QSize(0, 0))
        self.label_31.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_31.setFont(font)
        self.label_31.setObjectName(_fromUtf8("label_31"))
        self.gridLayout.addWidget(self.label_31, 10, 0, 1, 1)
        self.commandLinkButton = QtGui.QCommandLinkButton(self.dockWidgetContents)
        self.commandLinkButton.setMinimumSize(QtCore.QSize(0, 0))
        self.commandLinkButton.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.commandLinkButton.setObjectName(_fromUtf8("commandLinkButton"))
        self.gridLayout.addWidget(self.commandLinkButton, 10, 1, 1, 1)
        self.commandLinkButtonDIs = QtGui.QCommandLinkButton(self.dockWidgetContents)
        self.commandLinkButtonDIs.setEnabled(False)
        self.commandLinkButtonDIs.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.commandLinkButtonDIs.setObjectName(_fromUtf8("commandLinkButtonDIs"))
        self.gridLayout.addWidget(self.commandLinkButtonDIs, 10, 2, 1, 1)
        self.label_32 = QtGui.QLabel(self.dockWidgetContents)
        self.label_32.setMinimumSize(QtCore.QSize(0, 0))
        self.label_32.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_32.setFont(font)
        self.label_32.setObjectName(_fromUtf8("label_32"))
        self.gridLayout.addWidget(self.label_32, 11, 0, 1, 1)
        self.buttonBox = QtGui.QDialogButtonBox(self.dockWidgetContents)
        self.buttonBox.setMinimumSize(QtCore.QSize(0, 0))
        self.buttonBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.gridLayout.addWidget(self.buttonBox, 11, 1, 1, 1)
        self.buttonBoxDis = QtGui.QDialogButtonBox(self.dockWidgetContents)
        self.buttonBoxDis.setEnabled(False)
        self.buttonBoxDis.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBoxDis.setObjectName(_fromUtf8("buttonBoxDis"))
        self.gridLayout.addWidget(self.buttonBoxDis, 11, 2, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 4, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 12, 0, 1, 1)
        self.label_36 = QtGui.QLabel(self.dockWidgetContents)
        self.label_36.setAlignment(QtCore.Qt.AlignCenter)
        self.label_36.setObjectName(_fromUtf8("label_36"))
        self.gridLayout.addWidget(self.label_36, 13, 0, 1, 3)
        self.label = QtGui.QLabel(self.dockWidgetContents)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 9, 0, 1, 1)
        self.checkBoxTristate = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxTristate.setChecked(False)
        self.checkBoxTristate.setTristate(True)
        self.checkBoxTristate.setObjectName(_fromUtf8("checkBoxTristate"))
        self.gridLayout.addWidget(self.checkBoxTristate, 9, 1, 1, 1)
        self.checkBoxTristateDis = QtGui.QCheckBox(self.dockWidgetContents)
        self.checkBoxTristateDis.setEnabled(False)
        self.checkBoxTristateDis.setChecked(False)
        self.checkBoxTristateDis.setTristate(True)
        self.checkBoxTristateDis.setObjectName(_fromUtf8("checkBoxTristateDis"))
        self.gridLayout.addWidget(self.checkBoxTristateDis, 9, 2, 1, 1)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        QtCore.QObject.connect(self.radioButtonChecked, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.radioButtonCheckedDis.setChecked)
        QtCore.QObject.connect(self.radioButtonUnchecked, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.radioButtonUncheckedDis.setChecked)
        QtCore.QObject.connect(self.checkBoxChecked, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.checkBoxCheckedDis.setChecked)
        QtCore.QObject.connect(self.checkBoxEnabled, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.checkBoxUncheckedDis.setChecked)
        QtCore.QObject.connect(self.checkBoxTristate, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.checkBoxTristateDis.setChecked)
        QtCore.QObject.connect(self.commandLinkButton, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.commandLinkButtonDIs.setChecked)
        QtCore.QObject.connect(self.toolButton, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.toolButtonDis.setChecked)
        QtCore.QObject.connect(self.pushButtonChecked, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.pushButtonCheckedDis.setChecked)
        QtCore.QObject.connect(self.pushButtonUnchecked, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.pushButtonUncheckedDis.setChecked)
        QtCore.QObject.connect(self.pushButton, QtCore.SIGNAL(_fromUtf8("clicked(bool)")), self.pushButtonDis.click)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)

    def retranslateUi(self, DockWidget):
        DockWidget.setWindowTitle(_translate("DockWidget", "Buttons", None))
        self.label_72.setText(_translate("DockWidget", "Enabled", None))
        self.label_73.setText(_translate("DockWidget", "Disabled", None))
        self.label_26.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_26.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_26.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_26.setText(_translate("DockWidget", "PushButton", None))
        self.pushButton.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.pushButton.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.pushButton.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.pushButton.setText(_translate("DockWidget", "OK", None))
        self.pushButtonDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.pushButtonDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.pushButtonDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.pushButtonDis.setText(_translate("DockWidget", "OK", None))
        self.label_74.setText(_translate("DockWidget", "PushButton", None))
        self.pushButtonChecked.setText(_translate("DockWidget", "Checked", None))
        self.pushButtonCheckedDis.setText(_translate("DockWidget", "Checked", None))
        self.label_76.setText(_translate("DockWidget", "PushButton", None))
        self.pushButtonUnchecked.setText(_translate("DockWidget", "Unchecked", None))
        self.pushButtonUncheckedDis.setText(_translate("DockWidget", "Unchecked", None))
        self.label_33.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_33.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_33.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_33.setText(_translate("DockWidget", "ToolButton", None))
        self.toolButton.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.toolButton.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.toolButton.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.toolButton.setText(_translate("DockWidget", "Tool", None))
        self.toolButtonDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.toolButtonDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.toolButtonDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.toolButtonDis.setText(_translate("DockWidget", "Tool", None))
        self.label_75.setText(_translate("DockWidget", "RadioButton", None))
        self.radioButtonChecked.setText(_translate("DockWidget", "Checked", None))
        self.radioButtonCheckedDis.setText(_translate("DockWidget", "Checked", None))
        self.label_29.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_29.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_29.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_29.setText(_translate("DockWidget", "RadioButton", None))
        self.radioButtonUnchecked.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.radioButtonUnchecked.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.radioButtonUnchecked.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.radioButtonUnchecked.setText(_translate("DockWidget", "Unchecked", None))
        self.radioButtonUncheckedDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.radioButtonUncheckedDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.radioButtonUncheckedDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.radioButtonUncheckedDis.setText(_translate("DockWidget", "Unchecked", None))
        self.label_53.setText(_translate("DockWidget", "CheckBox", None))
        self.checkBoxChecked.setText(_translate("DockWidget", "Checked", None))
        self.checkBoxCheckedDis.setText(_translate("DockWidget", "Checked", None))
        self.label_30.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_30.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_30.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_30.setText(_translate("DockWidget", "CheckBox", None))
        self.checkBoxEnabled.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.checkBoxEnabled.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.checkBoxEnabled.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.checkBoxEnabled.setText(_translate("DockWidget", "Unchecked", None))
        self.checkBoxUncheckedDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.checkBoxUncheckedDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.checkBoxUncheckedDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.checkBoxUncheckedDis.setText(_translate("DockWidget", "Unchecked", None))
        self.label_31.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_31.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_31.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_31.setText(_translate("DockWidget", "CommandLinkButton", None))
        self.commandLinkButton.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.commandLinkButton.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.commandLinkButton.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.commandLinkButton.setText(_translate("DockWidget", "Command", None))
        self.commandLinkButtonDIs.setText(_translate("DockWidget", "Command", None))
        self.label_32.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_32.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_32.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_32.setText(_translate("DockWidget", "ButtonBox", None))
        self.buttonBox.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.buttonBox.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.buttonBox.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_36.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_36.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_36.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_36.setText(_translate("DockWidget", "Inside DockWidget", None))
        self.label.setText(_translate("DockWidget", "CheckBox", None))
        self.checkBoxTristate.setText(_translate("DockWidget", "Tristate", None))
        self.checkBoxTristateDis.setText(_translate("DockWidget", "Tristate", None))

