# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dw_inputs_no_fields.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName(_fromUtf8("DockWidget"))
        DockWidget.resize(402, 405)
        self.dockWidgetContents = QtGui.QWidget()
        self.dockWidgetContents.setObjectName(_fromUtf8("dockWidgetContents"))
        self.gridLayout = QtGui.QGridLayout(self.dockWidgetContents)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.dial = QtGui.QDial(self.dockWidgetContents)
        self.dial.setMinimumSize(QtCore.QSize(0, 0))
        self.dial.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.dial.setProperty("value", 50)
        self.dial.setObjectName(_fromUtf8("dial"))
        self.gridLayout.addWidget(self.dial, 2, 1, 1, 1)
        self.label_25 = QtGui.QLabel(self.dockWidgetContents)
        self.label_25.setMinimumSize(QtCore.QSize(0, 0))
        self.label_25.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_25.setFont(font)
        self.label_25.setObjectName(_fromUtf8("label_25"))
        self.gridLayout.addWidget(self.label_25, 7, 0, 1, 1)
        self.horizontalScrollBarDis = QtGui.QScrollBar(self.dockWidgetContents)
        self.horizontalScrollBarDis.setEnabled(False)
        self.horizontalScrollBarDis.setMinimumSize(QtCore.QSize(0, 0))
        self.horizontalScrollBarDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.horizontalScrollBarDis.setProperty("value", 50)
        self.horizontalScrollBarDis.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalScrollBarDis.setObjectName(_fromUtf8("horizontalScrollBarDis"))
        self.gridLayout.addWidget(self.horizontalScrollBarDis, 3, 2, 1, 1)
        self.verticalSlider = QtGui.QSlider(self.dockWidgetContents)
        self.verticalSlider.setMinimumSize(QtCore.QSize(0, 70))
        self.verticalSlider.setMaximumSize(QtCore.QSize(16777215, 70))
        self.verticalSlider.setProperty("value", 50)
        self.verticalSlider.setOrientation(QtCore.Qt.Vertical)
        self.verticalSlider.setObjectName(_fromUtf8("verticalSlider"))
        self.gridLayout.addWidget(self.verticalSlider, 7, 1, 1, 1)
        self.label_24 = QtGui.QLabel(self.dockWidgetContents)
        self.label_24.setMinimumSize(QtCore.QSize(0, 0))
        self.label_24.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_24.setFont(font)
        self.label_24.setObjectName(_fromUtf8("label_24"))
        self.gridLayout.addWidget(self.label_24, 4, 0, 1, 1)
        self.horizontalSlider = QtGui.QSlider(self.dockWidgetContents)
        self.horizontalSlider.setMinimumSize(QtCore.QSize(0, 0))
        self.horizontalSlider.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.horizontalSlider.setProperty("value", 50)
        self.horizontalSlider.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider.setObjectName(_fromUtf8("horizontalSlider"))
        self.gridLayout.addWidget(self.horizontalSlider, 4, 1, 2, 1)
        self.horizontalSliderDis = QtGui.QSlider(self.dockWidgetContents)
        self.horizontalSliderDis.setEnabled(False)
        self.horizontalSliderDis.setMinimumSize(QtCore.QSize(0, 0))
        self.horizontalSliderDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.horizontalSliderDis.setProperty("value", 50)
        self.horizontalSliderDis.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderDis.setObjectName(_fromUtf8("horizontalSliderDis"))
        self.gridLayout.addWidget(self.horizontalSliderDis, 4, 2, 1, 1)
        self.label_23 = QtGui.QLabel(self.dockWidgetContents)
        self.label_23.setMinimumSize(QtCore.QSize(0, 0))
        self.label_23.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setObjectName(_fromUtf8("label_23"))
        self.gridLayout.addWidget(self.label_23, 6, 0, 1, 1)
        self.label_2 = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 0, 2, 1, 1)
        self.verticalScrollBarDis = QtGui.QScrollBar(self.dockWidgetContents)
        self.verticalScrollBarDis.setEnabled(False)
        self.verticalScrollBarDis.setMinimumSize(QtCore.QSize(0, 70))
        self.verticalScrollBarDis.setMaximumSize(QtCore.QSize(16777215, 70))
        self.verticalScrollBarDis.setProperty("value", 50)
        self.verticalScrollBarDis.setOrientation(QtCore.Qt.Vertical)
        self.verticalScrollBarDis.setObjectName(_fromUtf8("verticalScrollBarDis"))
        self.gridLayout.addWidget(self.verticalScrollBarDis, 5, 2, 2, 1)
        self.label_21 = QtGui.QLabel(self.dockWidgetContents)
        self.label_21.setMinimumSize(QtCore.QSize(0, 0))
        self.label_21.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setObjectName(_fromUtf8("label_21"))
        self.gridLayout.addWidget(self.label_21, 2, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 8, 0, 1, 1)
        self.verticalScrollBar = QtGui.QScrollBar(self.dockWidgetContents)
        self.verticalScrollBar.setMinimumSize(QtCore.QSize(0, 70))
        self.verticalScrollBar.setMaximumSize(QtCore.QSize(16777215, 70))
        self.verticalScrollBar.setProperty("value", 50)
        self.verticalScrollBar.setOrientation(QtCore.Qt.Vertical)
        self.verticalScrollBar.setObjectName(_fromUtf8("verticalScrollBar"))
        self.gridLayout.addWidget(self.verticalScrollBar, 6, 1, 1, 1)
        self.comboBoxDis = QtGui.QComboBox(self.dockWidgetContents)
        self.comboBoxDis.setEnabled(False)
        self.comboBoxDis.setMinimumSize(QtCore.QSize(0, 0))
        self.comboBoxDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.comboBoxDis.setObjectName(_fromUtf8("comboBoxDis"))
        self.comboBoxDis.addItem(_fromUtf8(""))
        self.comboBoxDis.addItem(_fromUtf8(""))
        self.comboBoxDis.addItem(_fromUtf8(""))
        self.gridLayout.addWidget(self.comboBoxDis, 1, 2, 1, 1)
        self.horizontalScrollBar = QtGui.QScrollBar(self.dockWidgetContents)
        self.horizontalScrollBar.setMinimumSize(QtCore.QSize(0, 0))
        self.horizontalScrollBar.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.horizontalScrollBar.setProperty("value", 50)
        self.horizontalScrollBar.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalScrollBar.setObjectName(_fromUtf8("horizontalScrollBar"))
        self.gridLayout.addWidget(self.horizontalScrollBar, 3, 1, 1, 1)
        self.comboBox = QtGui.QComboBox(self.dockWidgetContents)
        self.comboBox.setMinimumSize(QtCore.QSize(0, 0))
        self.comboBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.comboBox.setObjectName(_fromUtf8("comboBox"))
        self.comboBox.addItem(_fromUtf8(""))
        self.comboBox.addItem(_fromUtf8(""))
        self.comboBox.addItem(_fromUtf8(""))
        self.gridLayout.addWidget(self.comboBox, 1, 1, 1, 1)
        self.label_22 = QtGui.QLabel(self.dockWidgetContents)
        self.label_22.setMinimumSize(QtCore.QSize(0, 0))
        self.label_22.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName(_fromUtf8("label_22"))
        self.gridLayout.addWidget(self.label_22, 3, 0, 1, 1)
        self.label = QtGui.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        self.label_50 = QtGui.QLabel(self.dockWidgetContents)
        self.label_50.setAlignment(QtCore.Qt.AlignCenter)
        self.label_50.setObjectName(_fromUtf8("label_50"))
        self.gridLayout.addWidget(self.label_50, 9, 0, 1, 3)
        self.label_11 = QtGui.QLabel(self.dockWidgetContents)
        self.label_11.setMinimumSize(QtCore.QSize(0, 0))
        self.label_11.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_11.setFont(font)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.gridLayout.addWidget(self.label_11, 1, 0, 1, 1)
        self.dialDis = QtGui.QDial(self.dockWidgetContents)
        self.dialDis.setEnabled(False)
        self.dialDis.setMinimumSize(QtCore.QSize(0, 0))
        self.dialDis.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.dialDis.setProperty("value", 50)
        self.dialDis.setObjectName(_fromUtf8("dialDis"))
        self.gridLayout.addWidget(self.dialDis, 2, 2, 1, 1)
        self.verticalSliderDis = QtGui.QSlider(self.dockWidgetContents)
        self.verticalSliderDis.setEnabled(False)
        self.verticalSliderDis.setMinimumSize(QtCore.QSize(0, 70))
        self.verticalSliderDis.setMaximumSize(QtCore.QSize(16777215, 70))
        self.verticalSliderDis.setProperty("value", 50)
        self.verticalSliderDis.setOrientation(QtCore.Qt.Vertical)
        self.verticalSliderDis.setObjectName(_fromUtf8("verticalSliderDis"))
        self.gridLayout.addWidget(self.verticalSliderDis, 7, 2, 1, 1)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        QtCore.QObject.connect(self.dial, QtCore.SIGNAL(_fromUtf8("sliderMoved(int)")), self.dialDis.setValue)
        QtCore.QObject.connect(self.horizontalScrollBar, QtCore.SIGNAL(_fromUtf8("sliderMoved(int)")), self.horizontalScrollBarDis.setValue)
        QtCore.QObject.connect(self.horizontalSlider, QtCore.SIGNAL(_fromUtf8("sliderMoved(int)")), self.horizontalSliderDis.setValue)
        QtCore.QObject.connect(self.verticalScrollBar, QtCore.SIGNAL(_fromUtf8("sliderMoved(int)")), self.verticalScrollBarDis.setValue)
        QtCore.QObject.connect(self.verticalSlider, QtCore.SIGNAL(_fromUtf8("sliderMoved(int)")), self.verticalSliderDis.setValue)
        QtCore.QObject.connect(self.comboBox, QtCore.SIGNAL(_fromUtf8("currentIndexChanged(int)")), self.comboBoxDis.setCurrentIndex)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)

    def retranslateUi(self, DockWidget):
        DockWidget.setWindowTitle(_translate("DockWidget", "Inputs - No Fields", None))
        self.dial.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.dial.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.dial.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_25.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_25.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_25.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_25.setText(_translate("DockWidget", "VerticalSlider", None))
        self.horizontalScrollBarDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.horizontalScrollBarDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.horizontalScrollBarDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.verticalSlider.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.verticalSlider.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.verticalSlider.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_24.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_24.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_24.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_24.setText(_translate("DockWidget", "HorizontalSlider", None))
        self.horizontalSlider.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.horizontalSlider.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.horizontalSlider.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.horizontalSliderDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.horizontalSliderDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.horizontalSliderDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_23.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_23.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_23.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_23.setText(_translate("DockWidget", "VerticalScroolBar", None))
        self.label_2.setText(_translate("DockWidget", "Disabled", None))
        self.verticalScrollBarDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.verticalScrollBarDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.verticalScrollBarDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_21.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_21.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_21.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_21.setText(_translate("DockWidget", "Dial", None))
        self.verticalScrollBar.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.verticalScrollBar.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.verticalScrollBar.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.comboBoxDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.comboBoxDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.comboBoxDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.comboBoxDis.setItemText(0, _translate("DockWidget", "ComboBox A", None))
        self.comboBoxDis.setItemText(1, _translate("DockWidget", "ComboBox B", None))
        self.comboBoxDis.setItemText(2, _translate("DockWidget", "ComboBox C", None))
        self.horizontalScrollBar.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.horizontalScrollBar.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.horizontalScrollBar.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.comboBox.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.comboBox.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.comboBox.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.comboBox.setItemText(0, _translate("DockWidget", "ComboBox A", None))
        self.comboBox.setItemText(1, _translate("DockWidget", "ComboBox B", None))
        self.comboBox.setItemText(2, _translate("DockWidget", "ComboBox C", None))
        self.label_22.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_22.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_22.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_22.setText(_translate("DockWidget", "HorizontalScroolBar", None))
        self.label.setText(_translate("DockWidget", "Enabled", None))
        self.label_50.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_50.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_50.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_50.setText(_translate("DockWidget", "Inside DockWidget", None))
        self.label_11.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.label_11.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.label_11.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.label_11.setText(_translate("DockWidget", "ComboBox", None))
        self.dialDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.dialDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.dialDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))
        self.verticalSliderDis.setToolTip(_translate("DockWidget", "This is a tool tip", None))
        self.verticalSliderDis.setStatusTip(_translate("DockWidget", "This is a status tip", None))
        self.verticalSliderDis.setWhatsThis(_translate("DockWidget", "This is \"what is this\"", None))

