// qra15_65_64_irr_e23.h
// Code tables and defines for Q-ary RA code (15,65) over GF(64)
// Code Name: qra15_65_64_irr_e23
// (15,65) RA Code over GF(64)

// (c) 2020 - Nico Palermo - IV3NWV - Microtelecom Srl, Italy

// This file is part of the qracodes project, a Forward Error Control
// encoding/decoding package based on Q-ary RA (Repeat and Accumulate) LDPC codes.
//
//    qracodes is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//    qracodes is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with qracodes source distribution.
//    If not, see <http://www.gnu.org/licenses/>.

#ifndef _qra15_65_64_irr_e23_h
#define _qra15_65_64_irr_e23_h

// File generated by npiwnarsavehc.m

#include "qracodes.h"

#ifdef __cplusplus
extern "C" {
#endif

extern const qracode qra15_65_64_irr_e23;

#ifdef __cplusplus
}
#endif

#endif // _qra15_65_64_irr_e23_h
